import SwiftUI
import Foundation
import SwiftData

@Model
final class ToDoItem {
    var timestamp: Date = Date()
    var task: String?
    var note: String?
    var due: Date?
    var completed: Bool = false
    var flagged: Bool = false
         
    init (task: String, note: String? = nil, due: Date? = nil) {
        self.task = task
        self.note = note
        self.due = due
    }
}

struct ContentView: View {
    @Environment(\.modelContext) private var modelContext
    @Query(sort: [SortDescriptor(\ToDoItem.due, order: .forward)]) private var toDoItems: [ToDoItem]
    @State private var sheetVisible = false
     
    var body: some View {
        NavigationView {
            List(toDoItems) { item in
                ToDoItemRow(
                    item: item,
                    toggleCompletion: { _ in toggleCompletion(item) },
                    flagItem: { _ in flagItem(item) },
                    deleteItem: { _ in deleteItem(item) }
                )
            }
            .navigationBarTitleDisplayMode(.inline)
            .navigationTitle("To-Do List")
            .toolbar {
                ToolbarItem(placement: .navigationBarTrailing) {
                    HStack {
                        Spacer()
                        Button(action: {
                            sheetVisible.toggle()
                        }) {
                            Image(systemName: "plus")
                        }
                        .sheet(isPresented: $sheetVisible) {
                            CreateTaskSheet()
                        }
                    }
                }
            }
        }
    }
     
    private func toggleCompletion(_ item: ToDoItem) {
        item.completed.toggle()
    }
     
    private func flagItem(_ item: ToDoItem) {
        item.flagged.toggle()
    }
     
    private func deleteItem(_ item: ToDoItem) {
        modelContext.delete(item)
    }
}


struct ToDoItemInfo: View {
    let item: ToDoItem
 
    var body: some View {
        HStack {
            if item.flagged {
                Image(systemName: "flag.fill")
                    .foregroundColor(.orange)
                    .padding(.trailing, 5)
            }
             
            VStack(alignment: .leading, spacing: 5) {
                Text(item.task ?? "No Title")
                    .font(.system(size: 20, weight: .bold))
                    .foregroundColor(item.completed ? .gray : .primary)
                    .strikethrough(item.completed, color: .gray)
                 
                Text(item.note ?? "")
                    .font(.system(size: 16, weight: .regular))
                    .foregroundColor(item.completed ? Color(UIColor.lightGray) : .gray)
                 
                if let dueDate = item.due {
                    let isOverdue = dueDate < Date()
                     
                    Text("Due: \(dueDate.formatted(.dateTime))")
                        .padding(4)
                        .background(isOverdue ? Color.red : Color.clear)
                        .foregroundColor(isOverdue ? .white : .primary)
                        .cornerRadius(8)
                        .font(.system(size: 12, weight: .regular))
                }
            }
        }
    }
}

struct ToDoItemRow: View {
    let item: ToDoItem
    let toggleCompletion: (ToDoItem) -> Void
    let flagItem: (ToDoItem) -> Void
    let deleteItem: (ToDoItem) -> Void
 
    var body: some View {
        HStack {
            ToDoItemInfo(item: item)
             
            Spacer()
             
            Button(action: {
                toggleCompletion(item)
            }) {
                Image(systemName: item.completed ? "checkmark.circle.fill" : "circle")
                    .resizable()
                    .frame(width: 24, height: 24)
                    .foregroundColor(item.completed ? .green : .gray)
            }
            .buttonStyle(PlainButtonStyle())
        }
        .padding(.vertical, 8)
        .swipeActions(edge: .leading) {
            Button {
                flagItem(item)
            } label: {
                Label("Flag", systemImage: "flag")
            }
            .tint(.orange)
        }
        .swipeActions {
            Button(role: .destructive) {
                deleteItem(item)
            } label: {
                Label("Delete", systemImage: "trash")
            }
            .tint(.red)
        }
    }
}

struct CreateTaskSheet: View {
    @Environment(\.modelContext) private var modelContext
    @Environment(\.dismiss) var dismiss
     
    @State private var todoItem = ""
    @State private var note = ""
     
    @State private var dueDate: Date = {
        let calendar = Calendar.current
        let now = Date()
        let today = calendar.startOfDay(for: now)
        let sevenPMToday = calendar.date(bySettingHour: 19, minute: 0, second: 0, of: today)!
         
        return now > sevenPMToday ? calendar.date(byAdding: .day, value: 1, to: sevenPMToday)! : sevenPMToday
    }()
     
    var body: some View {
        NavigationView {
            VStack(spacing: 20) {
                HStack {
                    Button("Cancel") {
                        dismiss()
                    }
                    .foregroundColor(.red)
                     
                    Spacer()
                     
                    Text("New Task")
                        .font(.title3)
                        .bold()
                     
                    Spacer()
                     
                    Button("Add") {
                        addItem()
                    }
                    .foregroundColor(.blue)
                    .disabled(todoItem.isEmpty)
                }
                .padding()
                .background(Color(UIColor.systemGray6))
                 
                VStack(alignment: .leading, spacing: 20) {
                    TextField("Task", text: $todoItem)
                        .textFieldStyle(RoundedBorderTextFieldStyle())
                        .padding(.horizontal)
                     
                    ZStack(alignment: .topLeading) {
                        if note.isEmpty {
                            Text("Add a note...")
                                .foregroundColor(.gray)
                                .padding(.horizontal, 8)
                                .padding(.vertical, 12)
                        }
                        TextEditor(text: $note)
                            .padding(8)
                            .background(Color(UIColor.systemGray6))
                            .cornerRadius(10)
                            .overlay(RoundedRectangle(cornerRadius: 10)
                                .stroke(Color.gray.opacity(0.5), lineWidth: 1))
                            .frame(height: 150)
                    }
                    .padding(.horizontal)
                     
                    VStack(alignment: .leading) {
                        Text("Due Date")
                            .font(.headline)
                            .padding(.leading)
                         
                        DatePicker(
                            "Select a date and time",
                            selection: $dueDate,
                            displayedComponents: [.date, .hourAndMinute]
                        )
                        .datePickerStyle(.compact)
                        .labelsHidden()
                        .padding(.horizontal)
                    }
                }
                .padding(.top, 20)
                 
                Spacer()
            }
            .background(Color(UIColor.systemBackground))
            .navigationBarTitleDisplayMode(.inline)
        }
    }
     
    private func addItem() {
        let newToDoItem = ToDoItem(task: todoItem, note: note, due: dueDate)
        modelContext.insert(newToDoItem)
        dismiss()
    }
}
 
struct CreateTaskSheet_Previews: PreviewProvider {
    static var previews: some View {
        CreateTaskSheet()
    }
}

#Preview {
    ContentView()
}
